@@ -2,7 +2,7 @@
 ## R source file
 ## This file is part of rgl
 ##
-## $Id: device.R,v 1.4 2004/05/28 07:05:25 dadler Exp $
+## $Id: device.R 376 2005-08-03 23:58:47Z dadler $
 ##
 
 ##
@@ -102,3 +102,23 @@ rgl.snapshot <- function( filename, fmt="png" )
     print("failed")
 }
 
+##
+## export postscript image
+##
+##
+
+rgl.postscript <- function( filename, fmt="eps" )
+{
+  idata <- as.integer(rgl.enum.gl2ps(fmt))
+
+  ret <- .C( symbol.C("rgl_postscript"),
+    success=FALSE,
+    idata,
+    as.character(filename),
+    PACKAGE="rgl"
+  )
+
+  if (! ret$success)
+    print("failed")
+}
+
